import { FASTElement } from "@microsoft/fast-element";
import { ListboxOption } from "../listbox-option/listbox-option";
import { ARIAGlobalStatesAndProperties } from "../patterns/aria-global";
/**
 * A Listbox Custom HTML Element.
 * Implements the {@link https://www.w3.org/TR/wai-aria-1.1/#listbox | ARIA listbox }.
 *
 * @public
 */
export declare class Listbox extends FASTElement {
    /**
     * The index of the selected option
     *
     * @public
     */
    selectedIndex: number;
    /**
     * Typeahead timeout in milliseconds.
     *
     * @internal
     */
    private static readonly TYPE_AHEAD_TIMEOUT_MS;
    /**
     * @internal
     */
    private typeaheadBuffer;
    /**
     * @internal
     */
    private typeaheadTimeout;
    /**
     * Flag for the typeahead timeout expiration.
     *
     * @internal
     */
    protected typeAheadExpired: boolean;
    /**
     * The role of the element.
     *
     * @public
     * @remarks
     * HTML Attribute: role
     */
    role: string;
    /**
     * The disabled state of the listbox.
     *
     * @public
     * @remarks
     * HTML Attribute: disabled
     */
    disabled: boolean;
    /**
     * @internal
     */
    options: ListboxOption[];
    optionsChanged(prev: any, next: any): void;
    /**
     * A collection of the selected options
     *
     * @public
     */
    selectedOptions: ListboxOption[];
    protected selectedOptionsChanged(prev: any, next: any): void;
    /**
     * @internal
     */
    get firstSelectedOption(): ListboxOption;
    /**
     * @internal
     */
    connectedCallback(): void;
    /**
     * @internal
     */
    protected focusAndScrollOptionIntoView(): void;
    /**
     * @internal
     */
    focusinHandler(e: FocusEvent): void;
    /**
     * @internal
     */
    private setDefaultSelectedOption;
    /**
     * Sets an option as selected and gives it focus.
     *
     * @param index - option index to select
     * @public
     */
    setSelectedOption(index?: number): void;
    /**
     * A static filter to include only enabled elements
     *
     * @param n - element to filter
     * @public
     */
    static slottedOptionFilter: (n: ListboxOption) => boolean;
    /**
     * Moves focus to the first selectable option
     *
     * @public
     */
    selectFirstOption(): void;
    /**
     * Moves focus to the last selectable option
     *
     * @internal
     */
    selectLastOption(): void;
    /**
     * Moves focus to the next selectable option
     *
     * @internal
     */
    selectNextOption(): void;
    /**
     * Moves focus to the previous selectable option
     *
     * @internal
     */
    selectPreviousOption(): void;
    /**
     * Handles click events for listbox options
     *
     * @internal
     */
    clickHandler(e: MouseEvent): boolean | void;
    /**
     * Handles keydown actions for listbox navigation and typeahead
     *
     * @internal
     */
    keydownHandler(e: KeyboardEvent): boolean | void;
    /**
     * Move focus to an option whose label matches characters typed by the user.
     * Consecutive keystrokes are batched into a buffer of search text used
     * to match against the set of options.  If TYPE_AHEAD_TIMEOUT_MS passes
     * between consecutive keystrokes, the search restarts.
     *
     * @param key - the key to be evaluated
     */
    handleTypeAhead(key: any): void;
}
/**
 * Includes ARIA states and properties relating to the ARIA button role
 *
 * @public
 */
export declare class DelegatesARIAListbox extends ARIAGlobalStatesAndProperties {
    /**
     * See {@link https://www.w3.org/WAI/PF/aria/roles#button} for more information
     * @public
     * @remarks
     * HTML Attribute: aria-pressed
     */
    ariaPressed: "true" | "false" | "mixed" | undefined;
    ariaActiveDescendant: string;
    /**
     * See {@link https://www.w3.org/WAI/PF/aria/roles#button} for more information
     * @public
     * @remarks
     * HTML Attribute: aria-expanded
     */
    ariaExpanded: "true" | "false" | undefined;
}
/**
 * @internal
 */
export interface Listbox extends DelegatesARIAListbox {
}
