import { __decorate } from "tslib";
import { uniqueId } from "lodash-es";
import { attr, DOM, FASTElement, observable } from "@microsoft/fast-element";
import { ARIAGlobalStatesAndProperties } from "../patterns/aria-global";
import { applyMixins } from "../utilities/apply-mixins";
import { ListboxRole } from "./listbox.options";
/**
 * A Listbox Custom HTML Element.
 * Implements the {@link https://www.w3.org/TR/wai-aria-1.1/#listbox | ARIA listbox }.
 *
 * @public
 */
export class Listbox extends FASTElement {
    constructor() {
        super(...arguments);
        /**
         * The index of the selected option
         *
         * @public
         */
        this.selectedIndex = -1;
        /**
         * @internal
         */
        this.typeaheadBuffer = "";
        /**
         * @internal
         */
        this.typeaheadTimeout = -1;
        /**
         * Flag for the typeahead timeout expiration.
         *
         * @internal
         */
        this.typeAheadExpired = true;
        /**
         * The role of the element.
         *
         * @public
         * @remarks
         * HTML Attribute: role
         */
        this.role = ListboxRole.listbox;
        /**
         * @internal
         */
        this.options = [];
        /**
         * A collection of the selected options
         *
         * @public
         */
        this.selectedOptions = [];
    }
    optionsChanged(prev, next) {
        if (this.$fastController.isConnected) {
            this.options.forEach(o => (o.id = o.id || uniqueId("option-")));
        }
    }
    selectedOptionsChanged(prev, next) {
        if (this.$fastController.isConnected) {
            this.options.forEach(o => (o.selected = next.includes(o)));
        }
    }
    /**
     * @internal
     */
    get firstSelectedOption() {
        return this.selectedOptions[0];
    }
    /**
     * @internal
     */
    connectedCallback() {
        super.connectedCallback();
        DOM.queueUpdate(() => this.setDefaultSelectedOption());
    }
    /**
     * @internal
     */
    focusAndScrollOptionIntoView() {
        if (this.contains(document.activeElement)) {
            this.firstSelectedOption.focus();
            this.firstSelectedOption.scrollIntoView({ block: "nearest" });
        }
    }
    /**
     * @internal
     */
    focusinHandler(e) {
        if (e.target === e.currentTarget) {
            this.setSelectedOption();
            this.focusAndScrollOptionIntoView();
        }
    }
    /**
     * @internal
     */
    setDefaultSelectedOption() {
        let selectedIndex = this.options.findIndex(el => el.selected);
        selectedIndex = selectedIndex !== -1 ? selectedIndex : 0;
        this.setSelectedOption(selectedIndex);
    }
    /**
     * Sets an option as selected and gives it focus.
     *
     * @param index - option index to select
     * @public
     */
    setSelectedOption(index = this.selectedIndex) {
        const selectedOption = this.options[index];
        if (!selectedOption) {
            return;
        }
        const selectedOptions = [];
        this.options.forEach(el => {
            if (el.isSameNode(selectedOption)) {
                selectedOptions.push(el);
            }
        });
        this.selectedIndex = index;
        this.selectedOptions = selectedOptions;
        this.ariaActiveDescendant = this.firstSelectedOption.id;
        this.focusAndScrollOptionIntoView();
    }
    /**
     * Moves focus to the first selectable option
     *
     * @public
     */
    selectFirstOption() {
        if (!this.disabled) {
            this.setSelectedOption(0);
        }
    }
    /**
     * Moves focus to the last selectable option
     *
     * @internal
     */
    selectLastOption() {
        if (!this.disabled) {
            this.setSelectedOption(this.options.length - 1);
        }
    }
    /**
     * Moves focus to the next selectable option
     *
     * @internal
     */
    selectNextOption() {
        if (!this.disabled && this.selectedIndex < this.options.length - 1) {
            this.setSelectedOption(this.selectedIndex + 1);
        }
    }
    /**
     * Moves focus to the previous selectable option
     *
     * @internal
     */
    selectPreviousOption() {
        if (!this.disabled && this.selectedIndex > 0) {
            this.setSelectedOption(this.selectedIndex - 1);
        }
    }
    /**
     * Handles click events for listbox options
     *
     * @internal
     */
    clickHandler(e) {
        const captured = e.target.closest(`[role=option]`);
        if (captured && !captured.disabled) {
            const selectedIndex = this.options.findIndex(el => el.isEqualNode(captured));
            this.setSelectedOption(selectedIndex);
            return true;
        }
    }
    /**
     * Handles keydown actions for listbox navigation and typeahead
     *
     * @internal
     */
    keydownHandler(e) {
        if (this.disabled) {
            return true;
        }
        const key = e.key || e.key.charCodeAt(0);
        switch (key) {
            // Select the first available option
            case "Home": {
                e.preventDefault();
                this.selectFirstOption();
                break;
            }
            // Select the next selectable option
            case "ArrowDown": {
                e.preventDefault();
                this.selectNextOption();
                break;
            }
            // Select the previous selectable option
            case "ArrowUp": {
                e.preventDefault();
                this.selectPreviousOption();
                break;
            }
            // Select the last available option
            case "End": {
                e.preventDefault();
                this.selectLastOption();
                break;
            }
            case "Tab": {
                this.focusAndScrollOptionIntoView();
                // fall through
            }
            case "Enter":
            case "Escape": {
                return true;
            }
            case " ": {
                if (this.typeAheadExpired) {
                    return true;
                }
            }
            // Send key to Typeahead handler
            default: {
                this.handleTypeAhead(key);
                return true;
            }
        }
    }
    /**
     * Move focus to an option whose label matches characters typed by the user.
     * Consecutive keystrokes are batched into a buffer of search text used
     * to match against the set of options.  If TYPE_AHEAD_TIMEOUT_MS passes
     * between consecutive keystrokes, the search restarts.
     *
     * @param key - the key to be evaluated
     */
    handleTypeAhead(key) {
        if (this.typeaheadTimeout) {
            window.clearTimeout(this.typeaheadTimeout);
        }
        this.typeaheadTimeout = window.setTimeout(() => (this.typeAheadExpired = true), Listbox.TYPE_AHEAD_TIMEOUT_MS);
        if (key.length > 1) {
            return;
        }
        if (this.typeAheadExpired) {
            this.typeaheadBuffer = "";
        }
        this.typeaheadBuffer += `${key}`;
        const pattern = `^(${this.typeaheadBuffer.replace(/[.*+\-?^${}()|[\]\\]/g, "\\$&")})`;
        const re = new RegExp(pattern, "gi");
        const selectedIndex = this.options.findIndex(o => o.text.trim().match(re));
        this.setSelectedOption(selectedIndex);
        this.typeAheadExpired = false;
    }
}
/**
 * Typeahead timeout in milliseconds.
 *
 * @internal
 */
Listbox.TYPE_AHEAD_TIMEOUT_MS = 1000;
/**
 * A static filter to include only enabled elements
 *
 * @param n - element to filter
 * @public
 */
Listbox.slottedOptionFilter = (n) => n.nodeType === Node.ELEMENT_NODE &&
    n.getAttribute("role") === "option" &&
    !n.disabled;
__decorate([
    attr
], Listbox.prototype, "role", void 0);
__decorate([
    attr({ mode: "boolean" })
], Listbox.prototype, "disabled", void 0);
__decorate([
    observable
], Listbox.prototype, "options", void 0);
__decorate([
    observable
], Listbox.prototype, "selectedOptions", void 0);
/**
 * Includes ARIA states and properties relating to the ARIA button role
 *
 * @public
 */
export class DelegatesARIAListbox extends ARIAGlobalStatesAndProperties {
    constructor() {
        super(...arguments);
        this.ariaActiveDescendant = "";
    }
}
__decorate([
    attr({ attribute: "aria-pressed", mode: "fromView" })
], DelegatesARIAListbox.prototype, "ariaPressed", void 0);
__decorate([
    observable
], DelegatesARIAListbox.prototype, "ariaActiveDescendant", void 0);
__decorate([
    observable
], DelegatesARIAListbox.prototype, "ariaExpanded", void 0);
applyMixins(Listbox, DelegatesARIAListbox);
