import { __decorate } from "tslib";
import { attr, DOM, observable, volatile } from "@microsoft/fast-element";
import { ARIAGlobalStatesAndProperties } from "../patterns/aria-global";
import { StartEnd } from "../patterns/start-end";
import { applyMixins } from "../utilities/apply-mixins";
import { SelectPosition, SelectRole } from "./select.options";
import { FormAssociatedSelect } from "./select.form-associated";
/**
 * A Select Custom HTML Element.
 * Implements the {@link https://www.w3.org/TR/wai-aria-1.1/#select | ARIA select }.
 *
 * @public
 */
export class Select extends FormAssociatedSelect {
    constructor() {
        super();
        /**
         * The open attribute.
         *
         * @internal
         */
        this.open = false;
        /**
         * Indicates the initial state of the position attribute.
         *
         * @internal
         */
        this.forcedPosition = false;
        /**
         * The role of the element.
         *
         * @public
         * @remarks
         * HTML Attribute: role
         */
        this.role = SelectRole.combobox;
        /**
         * Holds the current state for the calculated position of the listbox.
         *
         * @public
         */
        this.position = SelectPosition.below;
        /**
         * The max height for the listbox when opened.
         *
         * @internal
         */
        this.maxHeight = 0;
        this.value = this.initialValue;
    }
    openChanged() {
        this.ariaExpanded = this.open ? "true" : "false";
        if (this.open) {
            this.setPositioning();
            super.focusAndScrollOptionIntoView();
        }
    }
    /**
     * Calculate and apply listbox positioning based on available viewport space.
     *
     * @param force - direction to force the listbox to display
     * @public
     */
    setPositioning() {
        const currentBox = this.getBoundingClientRect();
        const viewportHeight = window.innerHeight;
        const availableBottom = viewportHeight - currentBox.bottom;
        this.position = this.forcedPosition
            ? this.positionAttribute
            : currentBox.top > availableBottom
                ? SelectPosition.above
                : SelectPosition.below;
        this.positionAttribute = this.forcedPosition
            ? this.positionAttribute
            : this.position;
        this.maxHeight =
            this.position === SelectPosition.above ? ~~currentBox.top : ~~availableBottom;
    }
    /**
     * The value displayed on the button.
     *
     * @public
     */
    get displayValue() {
        if (this.firstSelectedOption) {
            return this.firstSelectedOption.textContent || this.firstSelectedOption.value;
        }
        return "";
    }
    /**
     * Sets the value when the options are changed.
     *
     * @param prev - The previous value
     * @param next - the new value
     * @internal
     */
    selectedOptionsChanged(prev, next) {
        super.selectedOptionsChanged(prev, next);
        if (this.$fastController.isConnected) {
            this.value = this.firstSelectedOption.value;
        }
    }
    /**
     * @internal
     */
    clickHandler(e) {
        // do nothing if the select is disabled
        if (this.disabled) {
            return;
        }
        super.clickHandler(e);
        if (this.open) {
            const captured = e.target.closest(`[role=option]`);
            if (captured && captured.disabled) {
                return;
            }
        }
        this.open = !this.open;
        return true;
    }
    focusoutHandler(e) {
        if (!this.open) {
            return true;
        }
        const focusTarget = e.relatedTarget;
        if (this.isSameNode(focusTarget)) {
            DOM.queueUpdate(() => this.focus());
            return;
        }
        if (!this.options.includes(focusTarget)) {
            this.open = false;
        }
    }
    optionsChanged(prev, next) {
        super.optionsChanged(prev, next);
        this.setProxyOptions();
    }
    /**
     * Reset and fill the proxy to match the component's options
     * @internal
     */
    setProxyOptions() {
        if (this.proxy instanceof HTMLSelectElement) {
            this.proxy.options.length = 0;
            this.options.forEach(option => {
                if (option.proxy) {
                    this.proxy.appendChild(option.proxy);
                }
            });
        }
    }
    keydownHandler(e) {
        super.keydownHandler(e);
        const key = e.key || e.key.charCodeAt(0);
        switch (key) {
            case " ": {
                if (!this.typeAheadExpired) {
                    return true;
                }
                this.open = !this.open;
                return;
            }
            case "Enter": {
                this.open = !this.open;
                break;
            }
            case "Escape": {
                this.open = false;
                break;
            }
        }
        return true;
    }
    connectedCallback() {
        super.connectedCallback();
        this.setProxyOptions();
        this.forcedPosition = !!this.positionAttribute;
    }
}
__decorate([
    attr({ attribute: "open", mode: "boolean" })
], Select.prototype, "open", void 0);
__decorate([
    attr({ attribute: "position" })
], Select.prototype, "positionAttribute", void 0);
__decorate([
    attr
], Select.prototype, "role", void 0);
__decorate([
    observable
], Select.prototype, "position", void 0);
__decorate([
    observable
], Select.prototype, "maxHeight", void 0);
__decorate([
    volatile
], Select.prototype, "displayValue", null);
/**
 * Includes ARIA states and properties relating to the ARIA select role.
 *
 * @public
 */
export class DelegatesARIASelect extends ARIAGlobalStatesAndProperties {
}
__decorate([
    observable
], DelegatesARIASelect.prototype, "ariaExpanded", void 0);
applyMixins(Select, StartEnd, DelegatesARIASelect);
